@@ -54,6 +54,14 @@ def check_section(name, pattern, lines, proper_line, unique=True, start_index=0,
     return lines[0], content
 
 
+def is_float(s):
+    try:
+        float(s)
+        return True
+    except ValueError:
+        return False
+
+
 class ProblemService:
     @staticmethod
     def replace_image_urls(problem: Problem, from_url: str, to_url: str, replace_transaltions=True):
@@ -315,6 +323,7 @@ class ProblemService:
 
         all_files = sorted(glob.glob(os.path.join(problem_folder, "*.*")))
         solution_files = []
+        codestubs_files = []
         test_generator_files = []
 
         for file in all_files:
@@ -324,6 +333,8 @@ class ProblemService:
                 solution_files.append(file)
             elif "generator" in filename and file_ext in [".py", ".pas", ".cpp"]:
                 test_generator_files.append(file)
+            elif "codestub" in filename:
+                codestubs_files.append(file)
         # solution_files = sorted(glob.glob(os.path.join(problem_folder,
         #                                                f"*solution*\.(pas$|py$|cpp$)")))
         # solution_files += sorted(glob.glob(os.path.join(problem_folder,
@@ -342,7 +353,8 @@ class ProblemService:
             "testcases": testcase_file,
             "testcases_sample": testcase_sample_file,
             "test_generators": test_generator_files,
-            "solutions": solution_files
+            "solutions": solution_files,
+            "codestubs": codestubs_files
         }
 
         if not os.path.isfile(metadata_file) and not os.path.isfile(metadata_file_json):
@@ -754,6 +766,12 @@ class ProblemService:
                     problem.solution_lang = problem.solutions[0]["lang"]
                 # print(*problem.solutions, sep="\n")
 
+            if problem_files['codestubs']:
+                for file_path in problem_files['codestubs']:
+                    idx = file_path.rfind(".")
+                    with open(file_path, encoding="utf-8") as f:
+                        problem.code_stubs[file_path[idx + 1:]] = f.read()
+
             problem.testcase_generators = []
             if problem_files['test_generators']:
                 for file_path in problem_files['test_generators']:
@@ -768,6 +786,8 @@ class ProblemService:
             if force_testcase_format:
                 testcase_format = force_testcase_format
             ProblemService.load_testcases(problem_folder, testcase_format, problem)
+            if not problem.testcases:
+                problem.testcases = []
             print(f"Loaded {len(problem.testcases)} testcases")
 
         if not problem.testcases_sample and problem.testcases:
@@ -895,7 +915,7 @@ class ProblemService:
 
     @staticmethod
     def save(problem : Problem, base_folder=".", problem_folder=None, problem_code=None,
-             overwrite=False, lang=None, save_testcases=True, save_testcases_only=False):
+             overwrite=False, lang=None, save_testcases=True, save_testcases_only=False, testcase_format="ucode"):
         """
         :param problem:
         :param problem_code: name of problem folder
@@ -1031,9 +1051,19 @@ class ProblemService:
                     f.write(solution["code"])
                     f.close()
 
+            if problem.code_stubs:
+                for lang, code in problem.code_stubs.items():
+                    stub_file = f"codestub.{lang}"
+                    f = open(os.path.join(problem_folder, stub_file), 'w',
+                             encoding="utf-8", newline='')
+                    f.write(code)
+                    f.close()
+
         if save_testcases:
+            if not testcase_format:
+                testcase_format = problem.testcase_format
             from ucode.services.testcase.testcase_service import TestcaseService
-            TestcaseService.save_testcases(problem_folder, problem, format=problem.testcase_format)
+            TestcaseService.save_testcases(problem_folder, problem, format=testcase_format)
             # if save_testcases_only:
             #     CLog.warn("DON'T FORGET to change the 'testcase_format' to 'ucode' in meta data file.")
 
@@ -1125,6 +1155,38 @@ class ProblemService:
 
         return testcases
 
+    @staticmethod
+    def split_testcases_input(problem: Problem):
+        """
+        split all input number in data into separated lines, useful for primary problem (Scratch, python input)
+        @param problem:
+        @return:
+        """
+        for testcase in problem.testcases:
+            testcase.input = ProblemService.split_numbers(testcase.input)
+        for testcase in problem.testcases_sample:
+            testcase.input = ProblemService.split_numbers(testcase.input)
+        for testcase in problem.stock_testcases:
+            testcase.input = ProblemService.split_numbers(testcase.input)
+        for testcase in problem.stock_testcases_sample:
+            testcase.input = ProblemService.split_numbers(testcase.input)
+
+    @staticmethod
+    def split_numbers(input_data):
+        new_input = []
+        for line in input_data.splitlines():
+            tokens = line.split()
+            need_split = True
+            for t in tokens:
+                if not is_float(t):
+                    need_split = False
+                    break
+            if need_split:
+                new_input.extend(tokens)
+            else:
+                new_input.append(line)
+        return "\n".join(new_input)
+
 
 def convert_all_v1_to_new_format(src_folder, dest_folder=None, translations=[]):
     # problems = ProblemService.read_all_problems_v1(src_folder)
@@ -1168,8 +1230,21 @@ if __name__ == "__main__":
     # base_folder = "D:\\projects\\ucode\\dsa-thematic-problems\\competitive_programing_topics\\maths\\fundamentals\\gcd_lcm_new\\number_game"
     # ProblemService.convert(base_folder, ProblemFolderFormat.themis, ProblemFolderFormat.ucode,
     #                        convert_testcases_only=True, overwrite=True)
-    base_folder = r"D:\projects\ucode\weekly-algorithm-problems\competitions\teko-hcm\old"
-    convert_all_v1_to_new_format(base_folder)
+    # base_folder = r"D:\projects\ucode\weekly-algorithm-problems\competitions\teko-hcm\old"
+    # convert_all_v1_to_new_format(base_folder)
+
+    # problem = ProblemService.load(r"D:\projects\ucode\ucode-cli\problems\_save\quadrant_1")
+    # print(problem.code_stubs)
+
+    input_data = """6
+5 2 3
+100 1 4.2
+1 10 5s
+1000000000 1 6
+1 1 1000000000
+1 1 999999999"""
+    print(ProblemService.split_numbers(input_data))
+
     # DsaProblem.create_problem('../../problems', 'Counting Sort 3', lang="en",
     #                           translations=['vi', 'ru'],
     #                           overwrite=True,
