@@ -20,6 +20,16 @@ from ucode.services.dsa.problem_service import ProblemService
 _logger = logging.getLogger(__name__)
 
 
+class UCodeConstants:
+    ucode_judge_langs = {
+        "50": "c",
+        "54": "cpp",
+        "67": "pas",
+        "71": "py",
+        "101": "sb3"
+    }
+
+
 class UCode:
     def __init__(self, env="prod", domain="ucode.vn"):
         self.s = requests.session()
@@ -131,6 +141,94 @@ class UCode:
     def get_api_url(self, path):
         return self.api_base_url + path
 
+    def download_testcases(self, question_id, problem: Problem):
+        url = self.get_api_url(f"/question/{question_id}/testcase")
+        response = self.s.get(url, headers=self._headers)
+        print(url)
+        print("status_code:", response.status_code)
+        data_json = response.json()
+        if not data_json['success']:
+            CLog.error(f"Fail to get testcases of question #{question_id}: {json.dumps(data_json)}")
+            return []
+
+        testcase_json = data_json['data']
+        # print(json.dumps(testcase_json))
+
+        for i, test_data in enumerate(testcase_json):
+            CLog.info(f"Downloading testcase #{i+1}...")
+            input_data = requests.get(test_data['input_url']).text
+            output_data = requests.get(test_data['output_url']).text
+            test = TestCase(input=input_data, output=output_data, sample=test_data['is_sample'])
+            if test.sample:
+                problem.testcases_sample.append(test)
+            else:
+                problem.testcases.append(test)
+
+        return problem
+
+    def get_question(self, question_id, download_testcases=False, language=None):
+        url = self.get_api_url(f"/question/{question_id}")
+        if language:
+            url += f"?language={language}"
+        response = self.s.get(url, headers=self._headers)
+        print(url)
+        print("status_code:", response.status_code)
+        data_json = response.json()
+        if not data_json['success']:
+            CLog.error(f"Fail to get question #{question_id}: {json.dumps(data_json)}")
+            return
+
+        ques = data_json['data']
+        print(json.dumps(ques))
+
+        if ques['type'] in ['code']:
+            problem = Problem()
+            problem.name = ques['name']
+            problem.slug = make_slug(problem.name)
+            problem.statement_language = ques['statement_language']
+            if ques['code_stuff']:
+                code_stubs = {}
+                for lang, code in json.loads(ques['code_stuff']).items():
+                    ext = UCodeConstants.ucode_judge_langs.get(str(lang))
+                    if not ext:
+                        ext = lang
+                    code_stubs[ext] = code
+                problem.code_stubs = code_stubs
+                # print(problem.code_stubs)
+            problem.difficulty = ques['difficulty']
+            problem.tags = ques['tags']
+            problem.src = ques['source']
+            problem.src_url = ques['source_detail']
+            if not language and ques['languages']:
+                for lang in ques['languages']:
+                    if lang != problem.statement_language:
+                        problem.translations[lang] = \
+                            self.get_question(question_id, language=lang, download_testcases=False)
+            problem.statement = ques['statement']
+            problem.statement_format = ques['statement_format']
+            problem.input_format = ques['input_desc']
+            problem.output_format = ques['output_desc']
+            problem.constraints = ques['constraints']
+            if ques['hint']:
+                problem.editorial = json.loads(ques['hint']).get('text')
+
+            if ques['solutions']:
+                sol = json.loads(ques['solutions'])
+                sol = sol.get('text')
+                if sol:
+                    if "#include" in sol:
+                        lang = "cpp"
+                    elif "end." in sol:
+                        lang = "pas"
+                    else:
+                        lang = "py"
+                    problem.solutions = [{"lang": lang, "code": sol}]
+
+            if download_testcases:
+                self.download_testcases(question_id, problem)
+
+            return problem
+
     def add_question_to_quiz(self, question_ids: List[int], quiz_id: int):
         url = self.get_api_url(f"/lesson-item/{quiz_id}/add-question")
         payload = {
@@ -567,7 +665,7 @@ def create_problems_to_lesson_ucode(env="dev", domain="ucode.vn"):
     # base_folder = r"D:\projects\ucode\weekly-algorithm-problems\competitions\teko-hcm"
     # base_folder = r"D:\projects\freecontest\mock_tests\mock01"
     # base_folder = r"D:\projects\ucode\competitive_programing_topics\tests\sample_tests\thcs_testTHT"
-    base_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thcs_tests\test_02"
+    base_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thcs_tests\test_03"
 
     # base_folder = "D:\\projects\\ucode\\competitive_programing_topics\\Data_Structures\\Heap_PriorityQueue"
     problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
@@ -576,7 +674,7 @@ def create_problems_to_lesson_ucode(env="dev", domain="ucode.vn"):
     # prob_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thpt_tests\test_05\p04_bookshelves"
     # problems = [(prob_folder, ProblemService.load(prob_folder))]
 
-    lesson_id = 23465
+    lesson_id = 25478
     ucode = UCode(env=env, domain=domain)
     #
 
@@ -720,13 +818,27 @@ UX/UI	12/11/2020	BB #19: How to use Teko Design System for PO an BA	Trần Thàn
         # break
 
 
+def test_get_question():
+    env, domain = "prod", "ucode.vn"
+    question_id = 44242
+
+    ucode = UCode(env=env, domain=domain)
+
+    problem = ucode.get_question(question_id, download_testcases=True)
+
+    save_to = r"D:\projects\ucode\ucode-cli\problems\_save"
+    ProblemService.save(problem, save_to, overwrite=True, testcase_format="themis")
+
+
 if __name__ == "__main__":
     test_login()
     # teko_bb()
     # reupload_testcases()
 
     # create_problems_to_lesson_ucode("prod", domain="teko.ugrow.vn")
-    create_problems_to_lesson_ucode("prod")
+    # create_problems_to_lesson_ucode("prod")
+
+    test_get_question()
 
     # create_problem("dev")
     # create_problem("prod", domain="teko.ugrow.vn")
